@@ -1,4 +1,4 @@
-(def pdb-version "3.0.2")
+(def pdb-version "3.1.0")
 
 (defn deploy-info
   "Generate deployment information from the URL supplied and the username and
@@ -32,7 +32,6 @@
                  [org.clojure/math.combinatorics "0.0.4"]
                  [org.clojure/math.numeric-tower "0.0.4"]
                  [org.clojure/tools.logging "0.3.1"]
-                 [org.clojure/tools.nrepl "0.2.10"]
                  [puppetlabs/tools.namespace "0.2.4.1"]
                  [clj-stacktrace "0.2.8"]
                  [metrics-clojure "0.7.0" :exclusions [org.clojure/clojure org.slf4j/slf4j-api]]
@@ -41,7 +40,7 @@
                  ;; Filesystem utilities
                  [me.raynes/fs "1.4.5"]
                  ;; Version information
-                 [trptcolin/versioneer "0.1.0"]
+                 [puppetlabs/dujour-version-check "0.1.3"]
                  ;; Job scheduling
                  [overtone/at-at "1.2.0"]
                  ;; Nicer exception handling with try+/throw+
@@ -49,7 +48,7 @@
 
                  ;; Database connectivity
                  [com.jolbox/bonecp "0.7.1.RELEASE" :exclusions [org.slf4j/slf4j-api]]
-                 [org.clojure/java.jdbc "0.1.1"]
+                 [org.clojure/java.jdbc "0.3.7"]
                  [org.hsqldb/hsqldb "2.2.8"]
                  [org.postgresql/postgresql "9.2-1003-jdbc4"]
 
@@ -76,7 +75,8 @@
                  [robert/hooke "1.3.0"]
                  [honeysql "0.5.2"]
                  [org.clojure/data.xml "0.0.8"]
-                 [com.rpl/specter "0.5.7"]]
+                 [com.rpl/specter "0.5.7"]
+                 [org.clojure/core.async "0.1.346.0-17112a-alpha"]]
 
   :jvm-opts ["-XX:MaxPermSize=128M"]
 
@@ -102,7 +102,8 @@
   :lein-ezbake {:vars {:user "puppetdb"
                        :group "puppetdb"
                        :build-type "foss"
-                       :main-namespace "puppetlabs.puppetdb.main"}
+                       :main-namespace "puppetlabs.puppetdb.main"
+                       :repo-target "PC1"}
                 :config-dir "ext/config/foss"
                 }
 
@@ -125,7 +126,7 @@
              :ezbake {:dependencies ^:replace [[puppetlabs/puppetdb ~pdb-version]
                                                [org.clojure/tools.nrepl "0.2.3"]]
                       :name "puppetdb"
-                      :plugins [[puppetlabs/lein-ezbake "0.3.14"
+                      :plugins [[puppetlabs/lein-ezbake "0.3.18"
                                  :exclusions [org.clojure/clojure]]]}
              :testutils {:source-paths ^:replace ["test"]}
              :ci {:plugins [[lein-pprint "1.1.1"]]}}
