@@ -42,33 +42,33 @@
      data may linger in the database. We periodically sweep the
      database, compacting it and performing regular cleanup so we can
      maintain acceptable performance."
-  (:require [puppetlabs.puppetdb.scf.storage :as scf-store]
-            [puppetlabs.puppetdb.command.dlo :as dlo]
-            [puppetlabs.puppetdb.query.population :as pop]
-            [puppetlabs.puppetdb.jdbc :as pl-jdbc]
-            [puppetlabs.puppetdb.mq :as mq]
-            [clojure.java.jdbc :as sql]
+  (:require [clj-time.core :refer [ago]]
+            [clojure.java.io :as io]
+            [clojure.java.jdbc.deprecated :as sql]
             [clojure.tools.logging :as log]
-            [puppetlabs.puppetdb.http.server :as server]
-            [puppetlabs.puppetdb.config :as conf]
-            [puppetlabs.kitchensink.core :as kitchensink]
-            [robert.hooke :as rh]
-            [puppetlabs.trapperkeeper.core :refer [defservice] :as tk]
-            [puppetlabs.trapperkeeper.services :refer [service-id service-context]]
             [compojure.core :as compojure]
-            [clojure.java.io :refer [file]]
-            [clj-time.core :refer [ago]]
             [overtone.at-at :refer [mk-pool interspaced]]
-            [slingshot.slingshot :refer [throw+ try+]]
+            [puppetlabs.kitchensink.core :as kitchensink]
+            [puppetlabs.puppetdb.cheshire :as json]
+            [puppetlabs.puppetdb.command.constants :refer [command-names]]
+            [puppetlabs.puppetdb.command.dlo :as dlo]
+            [puppetlabs.puppetdb.config :as conf]
+            [puppetlabs.puppetdb.http.server :as server]
+            [puppetlabs.puppetdb.jdbc :as pl-jdbc :refer [with-transacted-connection]]
+            [puppetlabs.puppetdb.meta.version :as version]
+            [puppetlabs.puppetdb.scf.storage-utils :as sutils]
+            [puppetlabs.puppetdb.mq :as mq]
+            [puppetlabs.puppetdb.query-eng :as qeng]
+            [puppetlabs.puppetdb.query.population :as pop]
+            [puppetlabs.puppetdb.scf.migrate :refer [migrate! indexes!]]
+            [puppetlabs.puppetdb.scf.storage :as scf-store]
             [puppetlabs.puppetdb.time :refer [to-seconds to-millis parse-period
                                               format-period period?]]
-            [puppetlabs.puppetdb.jdbc :refer [with-transacted-connection]]
-            [puppetlabs.puppetdb.scf.migrate :refer [migrate! indexes!]]
-            [puppetlabs.puppetdb.meta.version :refer [version update-info]]
-            [puppetlabs.puppetdb.command.constants :refer [command-names]]
-            [puppetlabs.puppetdb.cheshire :as json]
-            [puppetlabs.puppetdb.query-eng :as qeng]
-            [puppetlabs.puppetdb.utils :as utils])
+            [puppetlabs.puppetdb.utils :as utils]
+            [puppetlabs.trapperkeeper.core :refer [defservice] :as tk]
+            [puppetlabs.trapperkeeper.services :refer [service-id service-context]]
+            [robert.hooke :as rh]
+            [slingshot.slingshot :refer [throw+ try+]])
   (:import [javax.jms ExceptionListener]))
 
 (def cli-description "Main PuppetDB daemon")
@@ -150,24 +150,10 @@
     (catch Exception e
       (log/error e "Error during garbage collection"))))
 
-(defn check-for-updates
-  "This will fetch the latest version number of PuppetDB and log if the system
-  is out of date."
-  [update-server db]
-  (let [{:keys [version newer link]} (try
-                                       (update-info update-server db)
-                                       (catch Throwable e
-                                         (log/debugf e "Could not retrieve update information (%s)" update-server)))]
-    (when newer
-      (log/info (cond-> (format "Newer version %s is available!" version)
-                  link (str " Visit " link " for details."))))))
-
 (defn maybe-check-for-updates
-  "Check for updates if our `product-name` indicates we should, and skip the
-  check otherwise."
-  [product-name update-server db]
+  [product-name update-server read-db]
   (if (= product-name "puppetdb")
-    (check-for-updates update-server db)
+    (version/check-for-updates! update-server read-db)
     (log/debug "Skipping update check on Puppet Enterprise")))
 
 (defn build-whitelist-authorizer
@@ -203,13 +189,8 @@
   "Shuts down PuppetDB, releasing resources when possible.  If this is
   not a normal shutdown, emergency? must be set, which currently just
   produces a fatal level level log message, instead of info."
-  [context emergency?]
-  (if emergency?
-    (log/error "A fatal error occurred; shutting down all subsystems.")
-    (log/info "Shutdown request received; puppetdb exiting."))
-  (when-let [updater (context :updater)]
-    (log/info "Shutting down updater thread.")
-    (future-cancel updater))
+  [context]
+  (log/info "Shutdown request received; puppetdb exiting.")
   (shutdown-mq context)
   (when-let [ds (get-in context [:shared-globals :scf-write-db :datasource])]
     (.close ds))
@@ -246,6 +227,7 @@
   [db-conn-pool product-name]
   (sql/with-connection db-conn-pool
     (scf-store/validate-database-version #(System/exit 1))
+    @sutils/db-metadata
     (migrate! db-conn-pool)
     (indexes! product-name)))
 
@@ -266,11 +248,9 @@
       (initialize-schema db-pool-map product-name))))
 
 (defn start-puppetdb
-  [context config service add-ring-handler get-route shutdown-on-error]
+  [context config service get-registered-endpoints]
   {:pre [(map? context)
-         (map? config)
-         (ifn? add-ring-handler)
-         (ifn? shutdown-on-error)]
+         (map? config)]
    :post [(map? %)
           (every? (partial contains? %) [:broker])]}
   (let [{:keys [global   jetty
@@ -284,16 +264,16 @@
                  max-frame-size threads]} command-processing
         {:keys [certificate-whitelist
                 disable-update-checking]} puppetdb
-        url-prefix (get-route service)
+
         write-db (pl-jdbc/pooled-datasource database)
         read-db (pl-jdbc/pooled-datasource (assoc read-database :read-only? true))
-        mq-dir (str (file vardir "mq"))
-        discard-dir (file mq-dir "discarded")
+        mq-dir (str (io/file vardir "mq"))
+        discard-dir (io/file mq-dir "discarded")
         mq-connection-str (add-max-framesize max-frame-size mq-addr)
         authorizer (when certificate-whitelist
                      (build-whitelist-authorizer certificate-whitelist))]
 
-    (when-let [v (version)]
+    (when-let [v (version/version)]
       (log/infof "PuppetDB version %s" v))
 
     (init-with-db database product-name)
@@ -322,24 +302,17 @@
                    :authorizer authorizer
                    :update-server update-server
                    :product-name product-name
-                   :url-prefix url-prefix
                    :discard-dir (.getAbsolutePath discard-dir)
                    :mq-addr mq-addr
                    :mq-dest mq-endpoint
                    :mq-threads threads
                    :catalog-hash-debug-dir catalog-hash-debug-dir
                    :command-mq {:connection mq-connection
-                                :endpoint mq-endpoint}}
-          updater (when-not disable-update-checking
-                    (future (shutdown-on-error
-                             (service-id service)
-                             #(maybe-check-for-updates product-name update-server read-db)
-                             #(stop-puppetdb % true))))]
+                                :endpoint mq-endpoint}}]
       (transfer-old-messages! mq-connection)
-      (let [app (->> (server/build-app globals)
-                     (compojure/context url-prefix []))]
-        (log/info "Starting query server")
-        (add-ring-handler service app))
+
+      (when-not disable-update-checking
+        (maybe-check-for-updates product-name update-server read-db))
 
       ;; Pretty much this helper just knows our job-pool and gc-interval
       (let [job-pool (mk-pool)
@@ -359,15 +332,15 @@
         ;; competition. Each task must handle its own errors.
         (gc-task db-maintenance-tasks)
         (gc-task #(compress-dlo! dlo-compression-threshold discard-dir)))
-      (-> context
-          (assoc :broker broker
-                 :mq-factory mq-factory
-                 :mq-connection mq-connection
-                 :shared-globals globals)
-          (merge (when updater {:updater updater}))))))
+      (assoc context
+             :broker broker
+             :mq-factory mq-factory
+             :mq-connection mq-connection
+             :shared-globals globals))))
 
 (defprotocol PuppetDBServer
   (shared-globals [this])
+  (set-url-prefix [this url-prefix])
   (query [this query-obj version query-expr paging-options row-callback-fn]
     "Call `row-callback-fn' for matching rows.  The `paging-options' should
     be a map containing :order_by, :offset, and/or :limit."))
@@ -378,18 +351,26 @@
   that trapperkeeper will call on exit."
   PuppetDBServer
   [[:ConfigService get-config]
-   [:WebroutingService add-ring-handler get-route]
-   [:ShutdownService shutdown-on-error]]
-
+   [:WebroutingService add-ring-handler get-registered-endpoints]]
+  (init [this context]
+        (assoc context :url-prefix (atom nil)))
   (start [this context]
-         (start-puppetdb context (get-config) this add-ring-handler get-route shutdown-on-error))
+         (start-puppetdb context (get-config) this get-registered-endpoints))
 
   (stop [this context]
-        (stop-puppetdb context false))
+        (stop-puppetdb context))
+  (set-url-prefix [this url-prefix]
+                  (let [old-url-prefix (:url-prefix (service-context this))]
+                    (when-not (compare-and-set! old-url-prefix nil url-prefix)
+                      (throw+ {:url-prefix old-url-prefix
+                               :new-url-prefix url-prefix
+                               :type ::url-prefix-already-set}
+                              (format "Attempt to set url-prefix to %s when it's already been set to %s" url-prefix @old-url-prefix)))))
   (shared-globals [this]
                   (:shared-globals (service-context this)))
   (query [this query-obj version query-expr paging-options row-callback-fn]
-         (let [{db :scf-read-db url-prefix :url-prefix} (get (service-context this) :shared-globals)]
+         (let [{db :scf-read-db} (get (service-context this) :shared-globals)
+               url-prefix @(get (service-context this) :url-prefix)]
            (qeng/stream-query-result query-obj version query-expr paging-options db url-prefix row-callback-fn))))
 
 (def ^{:arglists `([& args])
@@ -397,12 +378,7 @@
         TK's normal config parsing to do a bit of extra
         customization."}
   -main
-  (utils/wrap-main
-   (fn [& args]
-     (rh/add-hook #'puppetlabs.trapperkeeper.config/parse-config-data
-                  #'conf/hook-tk-parse-config-data)
-     (try+
-      (apply tk/main args)
-      (catch [:type ::conf/configuration-error] obj
-        (log/error (:message obj))
-        (throw+ (assoc obj ::utils/exit-status 1)))))))
+  (fn [& args]
+    (rh/add-hook #'puppetlabs.trapperkeeper.config/parse-config-data
+                 #'conf/hook-tk-parse-config-data)
+    (apply tk/main args)))
