@@ -69,11 +69,13 @@
             [slingshot.slingshot :refer [try+ throw+]]
             [cheshire.custom :refer [JSONable]]
             [puppetlabs.puppetdb.command.constants :refer [command-names]]
-            [puppetlabs.trapperkeeper.services :refer [defservice]]
+            [puppetlabs.trapperkeeper.services
+             :refer [defservice service-context]]
             [schema.core :as s]
             [puppetlabs.puppetdb.time :refer [to-timestamp]]
             [clj-time.core :refer [now]]
-            [clojure.set :as set]))
+            [clojure.set :as set]
+            [clojure.core.async :as async]))
 
 ;; ## Command parsing
 
@@ -114,36 +116,37 @@
 
 (defn annotate-command
   "Annotate a command-map with a timestamp and UUID"
-  [message]
+  [message uuid]
   {:pre  [(map? message)]
    :post [(map? %)]}
   (-> message
       (assoc-in [:annotations :received] (kitchensink/timestamp))
-      (assoc-in [:annotations :id] (kitchensink/uuid))))
+      (assoc-in [:annotations :id] (or uuid (kitchensink/uuid)))))
 
 ;; ## Command submission
 
-(defn-validated enqueue-raw-command! :- s/Str
+(defn-validated ^:private do-enqueue-raw-command :- s/Str
   "Submits raw-command to the mq-endpoint of mq-connection and returns
   its id."
   [mq-connection :- mq/connection-schema
    mq-endpoint :- s/Str
-   raw-command :- s/Str]
-  (let [id (kitchensink/uuid)]
-    (mq/send-message! mq-connection mq-endpoint
-                      raw-command
-                      {"received" (kitchensink/timestamp) "id" id})
-    id))
+   raw-command :- s/Str
+   uuid :- s/Str]
+  (mq/send-message! mq-connection mq-endpoint
+                    raw-command
+                    {"received" (kitchensink/timestamp) "id" uuid})
+  uuid)
 
-(defn-validated enqueue-command! :- s/Str
+(defn-validated ^:private do-enqueue-command :- s/Str
   "Submits command to the mq-endpoint of mq-connection and returns
   its id. Annotates the command via annotate-command."
   [mq-connection :- mq/connection-schema
    mq-endpoint :- s/Str
    command :- s/Str
    version :- s/Int
-   payload]
-  (let [command-map (annotate-command (assemble-command command version payload))]
+   payload
+   uuid :- (s/maybe s/Str)]
+  (let [command-map (annotate-command (assemble-command command version payload) uuid)]
     (mq/send-message! mq-connection mq-endpoint
                       (json/generate-string command-map))
     (get-in command-map [:annotations :id])))
@@ -257,14 +260,11 @@
   [{:keys [payload annotations]} {:keys [db]}]
   (let [certname (:certname payload)
         producer-timestamp (to-timestamp (:producer_timestamp payload (now)))
-        id  (:id annotations)
-        newer-record-exists? (fn [entity] (scf-storage/have-record-produced-after? entity certname producer-timestamp))]
+        id  (:id annotations)]
     (jdbc/with-transacted-connection db
       (when-not (scf-storage/certname-exists? certname)
         (scf-storage/add-certname! certname))
-      (if (not-any? newer-record-exists? [:catalogs :factsets :reports])
-        (scf-storage/deactivate-node! certname producer-timestamp)
-        (log/warnf "Not deactivating node %s because local data is newer than %s." certname producer-timestamp)))
+      (scf-storage/deactivate-node! certname producer-timestamp))
     (log/infof "[%s] [%s] %s" id (command-names :deactivate-node) certname)))
 
 ;; Report submission
@@ -285,6 +285,28 @@
                id (command-names :store-report)
                puppet_version certname)))
 
+(defn- resource->skipped-resource-events
+  "Fabricate a skipped resource-event"
+  [resource]
+  (-> resource
+      ;; We also need to grab the timestamp when the resource is `skipped'
+      (select-keys [:resource_type :resource_title :file :line :containment_path :timestamp])
+      (merge {:status "skipped" :property nil :old_value nil :new_value nil :message nil})
+      vector))
+
+(defn- resource->resource-events
+  [{:keys [skipped] :as resource}]
+  (if (= skipped true)
+    (resource->skipped-resource-events resource)
+    (let [resource-metadata (select-keys resource [:resource_type :resource_title :file :line :containment_path])]
+      (map (partial merge resource-metadata) (:events resource)))))
+
+(defn- resources->resource-events
+  [report]
+  (-> report
+      (update :resources (partial mapcat resource->resource-events))
+      (clojure.set/rename-keys {:resources :resource_events})))
+
 (defmethod process-command! [(command-names :store-report) 3]
   [{:keys [version] :as command} {:keys [db]}]
   (store-report* 5 db (-> command
@@ -299,14 +321,113 @@
   [{:keys [version] :as command} {:keys [db]}]
   (store-report* 5 db command))
 
+(defmethod process-command! [(command-names :store-report) 6]
+  [{:keys [version] :as command} {:keys [db]}]
+  (store-report* 6 db (update command :payload resources->resource-events)))
+
 (def supported-command?
   (comp (kitchensink/valset command-names) :command))
 
+(defprotocol PuppetDBCommandDispatcher
+  (enqueue-command
+    [this connection endpoint command version payload]
+    [this connection endpoint command version payload uuid]
+    "Annotates the command via annotate-command, submits it to the
+    endpoint of the connection, and then returns its unique id.")
+
+  (enqueue-raw-command [this connection endpoint raw-command uuid]
+    "Submits the raw-command to the endpoint of the connection and
+    returns the command's unique id.")
+
+  (stats [this]
+    "Returns command processing statistics as a map
+    containing :received-commands (a count of the commands received so
+    far by the current service instance), and :executed-commands (a
+    count of the commands that the current instance has processed
+    without triggering an exception).")
+
+  (response-mult [this]
+    "Returns a core.async mult to which {:id :exception} maps are written after
+     each command has been processed. " )
+
+  (response-pub [this]
+    "Returns a core.async pub to which {:id :exception} maps are written after
+     each command has been processed. The topic for each message is its
+     id (command uuid)."))
+
+(defn make-cmd-processed-message [cmd ex]
+  (merge {:id (-> cmd :annotations :id)}
+         (when ex {:exception ex})))
+
+(defn process-command-and-respond! [cmd config response-pub-chan stats-atom]
+  (try
+    (let [result (process-command! cmd config)]
+      (swap! stats-atom update :executed-commands inc)
+      (async/>!! response-pub-chan
+                 (make-cmd-processed-message cmd nil))
+      result)
+    (catch Exception ex
+      (async/>!! response-pub-chan
+                 (make-cmd-processed-message cmd ex))
+      (throw ex))))
+
 (defservice command-service
+  PuppetDBCommandDispatcher
   [[:PuppetDBServer shared-globals]
    [:MessageListenerService register-listener]]
+  (init [this context]
+    (let [response-chan (async/chan 1000)
+          response-mult (async/mult response-chan)
+          response-chan-for-pub (async/chan)]
+      (async/tap response-mult response-chan-for-pub)
+      (assoc context
+             :stats (atom {:received-commands 0
+                           :executed-commands 0})
+             :response-chan response-chan
+             :response-mult response-mult
+             :response-chan-for-pub response-chan-for-pub
+             :response-pub (async/pub response-chan-for-pub :id))))
+
   (start [this context]
-         (let [{:keys [scf-write-db catalog-hash-debug-dir]} (shared-globals)]
-           (register-listener supported-command? #(process-command! % {:db scf-write-db
-                                                                       :catalog-hash-debug-dir catalog-hash-debug-dir}))
-           context)))
+    (let [{:keys [scf-write-db catalog-hash-debug-dir]} (shared-globals)
+          config {:db scf-write-db
+                  :catalog-hash-debug-dir catalog-hash-debug-dir}
+          {:keys [response-chan response-pub]} context]
+      (register-listener
+       supported-command?
+       #(process-command-and-respond! % config response-chan (:stats context)))
+      context))
+
+  (stop [this context]
+    (async/unsub-all (:response-pub context))
+    (async/untap-all (:response-mult context))
+    (async/close! (:response-chan-for-pub context))
+    (async/close! (:response-chan context))
+    (dissoc context :response-pub :response-chan :response-chan-for-pub :response-mult))
+
+  (stats [this]
+    @(:stats (service-context this)))
+
+  (enqueue-command [this connection endpoint command version payload]
+    (enqueue-command this connection endpoint command version payload nil))
+
+  (enqueue-command [this connection endpoint command version payload uuid]
+    (let [result (do-enqueue-command connection endpoint
+                                     command version payload uuid)]
+
+
+      ;; Obviously assumes that if do-* doesn't throw, msg is in
+      (swap! (:stats (service-context this)) update :received-commands inc)
+      result))
+
+  (enqueue-raw-command [this connection endpoint raw-command uuid]
+    (let [result (do-enqueue-raw-command connection endpoint raw-command uuid)]
+      ;; Obviously assumes that if do-* doesn't throw, msg is in
+      (swap! (:stats (service-context this)) update :received-commands inc)
+      result))
+
+  (response-pub [this]
+    (-> this service-context :response-pub))
+
+  (response-mult [this]
+    (-> this service-context :response-mult)))
