@@ -1,6 +1,7 @@
 (ns puppetlabs.puppetdb.command-test
   (:require [me.raynes.fs :as fs]
-            [clojure.java.jdbc :as sql]
+            [clj-http.client :as client]
+            [clojure.java.jdbc.deprecated :as sql]
             [cheshire.core :as json]
             [puppetlabs.puppetdb.scf.storage :as scf-store]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
@@ -9,21 +10,26 @@
             [puppetlabs.puppetdb.scf.hash :as shash]
             [puppetlabs.trapperkeeper.testutils.logging :refer [atom-logger]]
             [clj-time.format :as tfmt]
+            [puppetlabs.puppetdb.cli.services :as cli-svc]
             [puppetlabs.puppetdb.command :refer :all]
             [puppetlabs.puppetdb.testutils :refer :all]
             [puppetlabs.puppetdb.fixtures :refer :all]
-            [puppetlabs.puppetdb.jdbc :refer [query-to-vec]]
+            [puppetlabs.puppetdb.jdbc :refer [query-to-vec] :as jdbc]
             [puppetlabs.puppetdb.examples :refer :all]
             [puppetlabs.puppetdb.testutils.reports :refer [munge-example-report-for-storage]]
+            [puppetlabs.puppetdb.testutils.services :as svc-utils]
             [puppetlabs.puppetdb.command.constants :refer [command-names]]
-            [clj-time.coerce :refer [to-timestamp to-date-time to-string]]
+            [clj-time.coerce
+             :refer [from-sql-date to-timestamp to-date-time to-string]]
             [clj-time.core :as t :refer [days ago now seconds]]
             [clojure.test :refer :all]
             [clojure.tools.logging :refer [*logger-factory*]]
             [slingshot.slingshot :refer [throw+]]
             [puppetlabs.puppetdb.mq-listener :as mql]
             [puppetlabs.puppetdb.utils :as utils]
-            [puppetlabs.puppetdb.time :as pt]))
+            [puppetlabs.puppetdb.time :as pt]
+            [puppetlabs.trapperkeeper.app :refer [get-service]])
+  (:import [org.joda.time DateTime DateTimeZone]))
 
 (use-fixtures :each with-test-db)
 
@@ -721,7 +727,7 @@
                                    cast(fv.value_float as text),
                                    '') as value,
                           fs.certname,
-                          e.name as environment,
+                          e.environment,
                           fs.producer_timestamp
                    FROM factsets fs
                      INNER JOIN facts as f on fs.id = f.factset_id
@@ -757,7 +763,7 @@
                                    cast(fv.value_float as text),
                                    '') as value,
                           fs.certname,
-                          e.name as environment
+                          e.environment
                    FROM factsets fs
                      INNER JOIN facts as f on fs.id = f.factset_id
                      INNER JOIN fact_values as fv on f.fact_value_id = fv.id
@@ -1147,14 +1153,32 @@
     (doseq [{:keys [certname command]} cases]
       (testing "should leave the node alone"
        (let [one-day   (* 24 60 60 1000)
-             yesterday (to-timestamp (- (System/currentTimeMillis) one-day))]
+             yesterday (to-timestamp (- (System/currentTimeMillis) one-day))
+             command (if (#{1 2} (:version command))
+                       ;; Can't set the :producer_timestamp for the older
+                       ;; versions (so that we can control the deactivation
+                       ;; timestamp).
+                       command
+                       (assoc-in command
+                                 [:payload :producer_timestamp] yesterday))]
          (sql/insert-record :certnames {:certname certname :deactivated yesterday})
-         (test-msg-handler command publish discard-dir
-           (is (= (query-to-vec "SELECT certname,deactivated FROM certnames")
-                  [{:certname certname :deactivated yesterday}]))
-           (is (= 0 (times-called publish)))
-           (is (empty? (fs/list-dir discard-dir)))
-           (sql/do-prepared "delete from certnames"))))))
+         (test-msg-handler
+           command
+           publish discard-dir
+           (let [[row & rest] (query-to-vec
+                               "SELECT certname,deactivated FROM certnames")]
+             (is (empty? rest))
+             (is (instance? java.sql.Timestamp (:deactivated row)))
+             (if (#{1 2} (:version command))
+               (do
+                 ;; Since we can't control the producer_timestamp.
+                 (is (= certname (:certname row)))
+                 (is (t/after? (from-sql-date (:deactivated row))
+                               (from-sql-date yesterday))))
+               (is (= {:certname certname :deactivated yesterday} row)))
+             (is (= 0 (times-called publish)))
+             (is (empty? (fs/list-dir discard-dir)))
+             (sql/do-prepared "delete from certnames")))))))
 
   (deftest deactivate-node-node-missing
     (testing "should add the node and deactivate it"
@@ -1178,6 +1202,33 @@
   "Report versions supported. Version 1 is not currently being tested."
   [:v5])
 
+(defn- resource-event->resource
+  [resource-event]
+  (-> resource-event
+      (select-keys [:file :line :timestamp :resource_type :resource_title :containment_path])
+      (assoc :skipped (= "skipped" (:status resource-event)))))
+
+(defn- resource-events->resources
+  [resource-events]
+  (for [[resource events] (group-by resource-event->resource resource-events)]
+    (assoc resource :events (map #(dissoc % :file :line :resource_type :resource_title :containment_path) events))))
+
+(let [report (-> (:basic report-examples/reports)
+                 (assoc :environment "DEV")
+                 munge-example-report-for-storage
+                 (update :resource_events resource-events->resources)
+                 (clojure.set/rename-keys {:resource_events :resources}))
+      v6-command {:command (command-names :store-report)
+                  :version 6
+                  :payload report}]
+  (deftest store-v6-report
+    (testing "should store the report"
+      (test-msg-handler v6-command publish discard-dir
+        (is (= [(with-env (select-keys report [:certname :configuration_version]))]
+               (query-to-vec "SELECT certname,configuration_version,environment_id FROM reports")))
+        (is (= 0 (times-called publish)))
+        (is (empty? (fs/list-dir discard-dir)))))))
+
 (let [report (-> (:basic report-examples/reports)
                  (assoc :environment "DEV")
                  munge-example-report-for-storage)
@@ -1246,6 +1297,75 @@
         (is (= 0 (times-called publish)))
         (is (empty? (fs/list-dir discard-dir)))))))
 
+(deftest command-service-stats
+  (svc-utils/with-puppetdb-instance
+    (let [pdb (get-service svc-utils/*server* :PuppetDBServer)
+          dispatcher (get-service svc-utils/*server* :PuppetDBCommandDispatcher)
+          shared-globals (partial cli-svc/shared-globals pdb)
+          enqueue-command (partial enqueue-command dispatcher)
+          stats (partial stats dispatcher)
+          real-replace! scf-store/replace-facts!
+          {{:keys [connection endpoint]} :command-mq} (shared-globals)]
+      ;; Issue a single command and ensure the stats are right at each step.
+      (is (= {:received-commands 0 :executed-commands 0} (stats)))
+      (let [received-cmd? (promise)
+            go-ahead-and-execute (promise)]
+        (with-redefs [scf-store/replace-facts!
+                      (fn [& args]
+                        (deliver received-cmd? true)
+                        @go-ahead-and-execute
+                        (apply real-replace! args))]
+          (enqueue-command connection endpoint
+                           (command-names :replace-facts) 4
+                           {:environment "DEV" :certname "foo.local"
+                            :values {:foo "foo"}
+                            :producer_timestamp (to-string (now))})
+          @received-cmd?
+          (is (= {:received-commands 1 :executed-commands 0} (stats)))
+          (deliver go-ahead-and-execute true)
+          (while (not= 1 (:executed-commands (stats)))
+            (Thread/sleep 100))
+          (is (= {:received-commands 1 :executed-commands 1} (stats))))))))
+
+(deftest date-round-trip
+  (svc-utils/with-puppetdb-instance
+    (let [pdb (get-service svc-utils/*server* :PuppetDBServer)
+          dispatcher (get-service svc-utils/*server* :PuppetDBCommandDispatcher)
+          shared-globals (partial cli-svc/shared-globals pdb)
+          enqueue-command (partial enqueue-command dispatcher)
+          globals (shared-globals)
+          {{:keys [connection endpoint]} :command-mq} globals
+          deactivate-ms 14250331086887
+          ;; The problem only occurred if you passed a Date to
+          ;; enqueue, a DateTime wasn't a problem.
+          input-stamp (java.util.Date. deactivate-ms)
+          expected-stamp (DateTime. deactivate-ms DateTimeZone/UTC)]
+      (enqueue-command connection endpoint
+                       (command-names :deactivate-node) 3
+                       {:certname "foo.local" :producer_timestamp input-stamp})
+      (is (svc-utils/wait-for-server-processing svc-utils/*server* 5000))
+      ;; While we're here, check the value in the database too...
+      (is (= expected-stamp
+             (jdbc/with-transacted-connection
+               (:scf-read-db globals)
+               :repeatable-read
+               (from-sql-date (scf-store/node-deactivated-time "foo.local")))))
+      (is (= expected-stamp
+             (-> (client/get (str (utils/base-url->str svc-utils/*base-url*)
+                                  "/nodes")
+                             {:accept :json
+                              :throw-exceptions true
+                              :throw-entire-message true
+                              :query-params {"query"
+                                             (json/generate-string
+                                              ["or" ["=" ["node" "active"] true]
+                                               ["=" ["node" "active"] false]])}})
+                 :body
+                 json/parse-string
+                 first
+                 (get "deactivated")
+                 (pt/from-string)))))))
+
 ;; Local Variables:
 ;; mode: clojure
 ;; eval: (define-clojure-indent (test-msg-handler (quote defun))
