@@ -63,7 +63,7 @@
      :partition-count (pls/defaulted-maybe s/Int 1)
      :stats (pls/defaulted-maybe String "true")
      :log-statements (pls/defaulted-maybe String "true")
-     :statements-cache-size (pls/defaulted-maybe s/Int 0)
+     :statements-cache-size (pls/defaulted-maybe s/Int 1000)
      :connection-timeout (pls/defaulted-maybe s/Int 500)}))
 
 (def write-database-config-in
@@ -159,7 +159,7 @@
    :subname     (format "file:%s;hsqldb.tx=mvcc;sql.syntax_pgs=true" (io/file vardir "db"))})
 
 (defn validate-db-settings
-  "Throws a {:type ::configuration-error :message m} exception
+  "Throws a {:type ::cli-error :message m} exception
   describing the required additions if the [database] configuration
   doesn't specify classname, subprotocol and subname, all of which are
   now required."
@@ -179,7 +179,8 @@
                    (setting-needed :classname)
                    (setting-needed :subprotocol)
                    (setting-needed :subname))]
-      (throw+ {:type ::configuration-error :message msg})))
+      (throw+ {:type ::cli-error
+               :message msg})))
   config)
 
 (defn convert-section-config
@@ -334,12 +335,8 @@
 
 (defn add-web-routing-service-config
   [config-data]
-  (let [default-web-router-service
-        {:puppetlabs.puppetdb.cli.services/puppetdb-service "/pdb/query"
-         :puppetlabs.puppetdb.http.command/puppetdb-command-service "/pdb/cmd"
-         :puppetlabs.puppetdb.meta/metadata-service "/pdb/meta"
-         :puppetlabs.puppetdb.dashboard/dashboard-service "/pdb"
-         :puppetlabs.puppetdb.metrics/metrics-service "/metrics"}
+  (let [default-web-router-service {:puppetlabs.puppetdb.metrics/metrics-service "/metrics"
+                                    :puppetlabs.puppetdb.pdb-routing/pdb-routing-service "/pdb"}
         bootstrap-cfg (-> (find-bootstrap-config config-data)
                           slurp
                           str/split-lines)
@@ -365,7 +362,7 @@
 (defn hook-tk-parse-config-data
   "This is a robert.hooke compatible hook that is designed to intercept
    trapperkeeper configuration before it is used, so that we may munge &
-   customize it.  It may throw {:type ::configuration-error :message m}."
+   customize it.  It may throw {:type ::cli-error :message m}."
   [f args]
   (adjust-and-validate-tk-config (f args)))
 
