@@ -1,19 +1,21 @@
 (ns puppetlabs.puppetdb.fixtures
   (:import [java.io ByteArrayInputStream])
-  (:require [clojure.java.jdbc :as sql]
-            [clojure.java.jdbc.internal :as jint]
+  (:require [clojure.java.jdbc.deprecated :as sql]
             [puppetlabs.puppetdb.http.server :as server]
+            [puppetlabs.puppetdb.command :as dispatch]
             [puppetlabs.puppetdb.http.command :as command]
             [puppetlabs.puppetdb.jdbc :as pjdbc]
             [puppetlabs.puppetdb.schema :as pls]
             [puppetlabs.puppetdb.config :as cfg]
+            [puppetlabs.puppetdb.scf.storage-utils :as sutils]
             [puppetlabs.trapperkeeper.testutils.logging :refer [with-test-logging]]
             [clojure.tools.macro :as tmacro]
             [clojure.test :refer [join-fixtures use-fixtures]]
             [puppetlabs.puppetdb.testutils
              :refer [clear-db-for-testing! test-db with-test-broker without-jmx]]
             [puppetlabs.trapperkeeper.logging :refer [reset-logging]]
-            [puppetlabs.puppetdb.scf.migrate :refer [migrate!]]))
+            [puppetlabs.puppetdb.scf.migrate :refer [migrate!]]
+            [puppetlabs.puppetdb.middleware :as mid]))
 
 (def ^:dynamic *db* nil)
 (def ^:dynamic *mq* nil)
@@ -29,14 +31,23 @@
          (migrate! *db*)))
       (pjdbc/pooled-datasource (assoc db :read-only? read-only?)))))
 
+(defn with-db-metadata
+  "A fixture to collect DB type and version information before a test."
+  [f]
+  (binding [*db* (test-db)]
+    (sql/with-connection *db*
+      (with-redefs [sutils/db-metadata (delay (sutils/db-metadata-fn))]
+        (f)))))
+
 (defn with-test-db
   "A fixture to start and migrate a test db before running tests."
   [f]
   (binding [*db* (test-db)]
     (sql/with-connection *db*
-      (clear-db-for-testing!)
-      (migrate! *db*)
-      (f))))
+      (with-redefs [sutils/db-metadata (delay (sutils/db-metadata-fn))]
+        (clear-db-for-testing!)
+        (migrate! *db*)
+        (f)))))
 
 (defn without-db-var
   "Binds the java.jdbc dtabase connection to nil. When running a unit
@@ -47,7 +58,7 @@
    route testing code to ensure that the route has it's own db
    connection."
   [f]
-  (binding [jint/*db* nil]
+  (binding [sql/*db* nil]
     (f)))
 
 (defn with-test-mq
@@ -66,7 +77,13 @@
   is available. Note this means this fixture should be nested _within_
   `with-test-mq`."
   ([f]
-   (binding [*command-app* (command/command-app {:command-mq *mq*})]
+   (binding [*command-app* (mid/wrap-with-puppetdb-middleware
+                            (command/command-app
+                             (fn [] *mq*)
+                             (fn [] {})
+                             #'dispatch/do-enqueue-raw-command
+                             (fn [] nil))
+                            nil)]
      (f))))
 
 (defn with-http-app
@@ -75,17 +92,18 @@
   are available. Note this means this fixture should be nested _within_
   `with-test-db` or `with-test-mq`."
   ([f]
-     (with-http-app {} f))
+   (with-http-app {} f))
   ([globals-overrides f]
-     (binding [*app* (server/build-app
-                       (merge
-                         {:scf-read-db          *db*
-                          :scf-write-db         *db*
-                          :command-mq           *mq*
-                          :product-name         "puppetdb"
-                          :url-prefix           ""}
-                         globals-overrides))]
-       (f))))
+   (let [get-shared-globals #(merge {:scf-read-db *db*
+                                     :scf-write-db *db*
+                                     :command-mq *mq*
+                                     :product-name "puppetdb"
+                                     :url-prefix ""}
+                                    globals-overrides)]
+     (binding [*app* (mid/wrap-with-puppetdb-middleware
+                      (server/build-app get-shared-globals)
+                      nil)]
+       (f)))))
 
 (defn defaulted-write-db-config
   "Defaults and converts `db-config` from the write database INI format to the internal
