@@ -135,13 +135,21 @@
                                ["not" ["=" "certname" (:certname basic)]]])
        #{{:hash bar-report-hash}}))
 
-    (testing "projected aggregate function call"
+    (testing "projected aggregate count call"
       (response-equal?
         (get-response endpoint ["extract" [["function" "count"] "status"]
                                 ["~" "certname" ".*"]
                                 ["group_by" "status"]])
         #{{:status "unchanged", :count 2}}))
 
+    (testing "projected aggregate sum call"
+      (response-equal?
+        (get-response endpoint ["extract"
+                                [["function" "sum" "report_format"] "status"]
+                                ["~" "certname" ".*"]
+                                ["group_by" "status"]])
+        #{{:status "unchanged", :sum 8}}))
+
     (testing "projected aggregate function call with two column groupings"
       (response-equal?
         (get-response endpoint ["extract" [["function" "count"] "status" "certname"]
