@@ -24,7 +24,7 @@
     (let [ops (query/resource-event-ops version)]
       (testing "should succesfully compile a valid equality query"
         (is (= (query/compile-term ops ["=" "report" "blah"])
-               {:where   (format "%s = ?" (sutils/sql-hash-as-str "reports.hash"))
+               {:where   (format "%s = ?" (sutils/sql-hash-as-str "latest_events.hash"))
                 :params  ["blah"]})))
       (testing "should fail with an invalid equality query"
         (is (thrown-with-msg?
