@@ -1,5 +1,5 @@
 (ns puppetlabs.puppetdb.query-eng
-  (:require [clojure.java.jdbc :as sql]
+  (:require [clojure.java.jdbc.deprecated :as sql]
             [clojure.tools.logging :as log]
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.puppetdb.cheshire :as json]
@@ -127,7 +127,7 @@
                                   LIMIT 1")
                     :environment "SELECT 1
                                   FROM environments
-                                  WHERE name=?"
+                                  WHERE environment=?"
                     :factset "SELECT 1
                               FROM certnames
                               INNER JOIN factsets
