@@ -25,7 +25,7 @@
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
             [puppetlabs.puppetdb.jdbc :as jdbc]
-            [clojure.java.jdbc :as sql]
+            [clojure.java.jdbc.deprecated :as sql]
             [clojure.set :as set]
             [clojure.string :as str]
             [clojure.tools.logging :as log]
@@ -85,7 +85,7 @@
 
 (def environments-schema
   {:id s/Int
-   :name s/Str})
+   :environment s/Str})
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas - Internal
@@ -296,7 +296,7 @@
 (pls/defn-validated environment-id :- (s/maybe s/Int)
   "Returns the id (primary key) from the environments table for the given `env-name`"
   [env-name :- s/Str]
-  (query-id :environments {:name env-name}))
+  (query-id :environments {:environment env-name}))
 
 (pls/defn-validated certname-id :- (s/maybe s/Int)
   [certname :- s/Str]
@@ -307,7 +307,7 @@
    the id of the `env-name` (whether created or existing)"
   [env-name :- (s/maybe s/Str)]
   (when env-name
-    (ensure-row :environments {:name env-name})))
+    (ensure-row :environments {:environment env-name})))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Status querying/updating
@@ -1218,6 +1218,34 @@
   [certname :- String]
   (sql/insert-record :certnames {:certname certname}))
 
+(defn timestamp-of-newest-record [entity certname]
+  (let [query {:select [:producer_timestamp]
+               :from [entity]
+               :where [:= :certname certname]
+               :order-by [[:producer_timestamp :desc]]
+               :limit 1}]
+    (sql/with-query-results result-set (hcore/format query)
+      (:producer_timestamp (first result-set)))))
+
+(pls/defn-validated have-record-produced-after?
+  [entity :- s/Keyword
+   certname :- String
+   time :- pls/Timestamp]
+  (let [time (to-timestamp time)]
+    (boolean
+     (some-> entity
+             (timestamp-of-newest-record certname)
+             (.after time)))))
+
+(pls/defn-validated have-newer-record-for-certname?
+  [certname :- String
+   timestamp :- pls/Timestamp]
+  "Returns a truthy value indicating whether a record exists that has
+  a producer_timestamp newer than the given timestamp."
+  (some (fn [entity]
+          (have-record-produced-after? entity certname timestamp))
+        [:catalogs :factsets :reports]))
+
 (pls/defn-validated maybe-activate-node!
   "Reactivate the given host, only if it was deactivated or expired before
   `time`.  Returns true if the node is activated, or if it was already active.
@@ -1237,11 +1265,17 @@
 (pls/defn-validated deactivate-node!
   "Deactivate the given host, recording the current time. If the node is
   currently inactive, no change is made."
-  [certname :- String & [timestamp :- pls/Timestamp]]
-  (let [timestamp (to-timestamp (or timestamp (now)))]
-   (sql/do-prepared "UPDATE certnames SET deactivated = ?
-                    WHERE certname=? AND deactivated IS NULL"
-                    [timestamp certname])))
+  ([certname :- String]
+   (deactivate-node! certname (now)))
+  ([certname :- String timestamp :- pls/Timestamp]
+   (if (have-newer-record-for-certname? certname timestamp)
+     (log/warnf "Not deactivating node %s because local data is newer than %s."
+                certname timestamp)
+     (let [sql-timestamp (to-timestamp timestamp)]
+       (sql/do-prepared "UPDATE certnames SET deactivated = ?
+                           WHERE certname=?
+                             AND (deactivated IS NULL OR deactivated < ?)"
+                        [sql-timestamp certname sql-timestamp])))))
 
 (pls/defn-validated expire-node!
   "Expire the given host, recording the current time. If the node is
@@ -1252,25 +1286,6 @@
                     WHERE certname=? AND expired IS NULL"
                     [timestamp certname])))
 
-(defn timestamp-of-newest-record [entity certname]
-  (let [query {:select [:producer_timestamp]
-               :from [entity]
-               :where [:= :certname certname]
-               :order-by [[:producer_timestamp :desc]]
-               :limit 1}]
-    (sql/with-query-results result-set (hcore/format query)
-      (:producer_timestamp (first result-set)))))
-
-(pls/defn-validated have-record-produced-after?
-  [entity :- s/Keyword
-   certname :- String
-   time :- pls/Timestamp]
-  (let [time (to-timestamp time)]
-    (boolean
-     (some-> entity
-             (timestamp-of-newest-record certname)
-             (.after time)))))
-
 (pls/defn-validated catalog-newer-than?
   "Returns true if the most current catalog for `certname` is more recent than
   `time`."
