@@ -1,5 +1,5 @@
 (ns puppetlabs.puppetdb.scf.storage-test
-  (:require [clojure.java.jdbc :as sql]
+  (:require [clojure.java.jdbc.deprecated :as sql]
             [puppetlabs.puppetdb.cheshire :as json]
             [puppetlabs.puppetdb.scf.hash :as shash]
             [puppetlabs.puppetdb.facts :as facts
@@ -1158,7 +1158,12 @@
       (testing "should not change the node if it's already inactive"
         (let [original (query-certnames)]
           (deactivate-node! certname)
-          (is (= original (query-certnames))))))
+          ;; Convert any :deactivated values to #t for comparison
+          ;; since we only care about the state.
+          (letfn [(deactivated->truthy [x]
+                    (assoc x :deactivated (when (:deactivated x) true)))]
+            (is (= (map deactivated->truthy original)
+                   (map deactivated->truthy (query-certnames))))))))
 
     (testing "activating a node"
       (testing "should activate the node if it was inactive"
@@ -1393,8 +1398,8 @@
         (is (= expected actual))))))
 
 (defn with-db-version [db version f]
-  (with-redefs-fn {#'sutils/sql-current-connection-database-name (constantly db)
-                   #'sutils/sql-current-connection-database-version (constantly version)}
+  (with-redefs [sutils/db-metadata (delay {:database db
+                                           :version version})]
     f))
 
 (deftest test-db-unsupported-msg
